import React from 'react';
import {Box, Typography, Button, Stack, TextField} from '@mui/material';
import QRCode from 'react-qr-code';
import DeleteIcon from "@mui/icons-material/Delete";
import {Dewar, Shipment_Input, DefaultService} from "../../openapi";
import {SxProps} from "@mui/system";
import CustomStepper from "./DewarStepper";
import DewarDetails from './DewarDetails';



interface ShipmentDetailsProps {
    isCreatingShipment: boolean;
    selectedShipment: Shipment_Input;
    selectedDewar: Dewar | null;
    setSelectedDewar: React.Dispatch<React.SetStateAction<Dewar | null>>;
    sx?: SxProps;
}

const ShipmentDetails: React.FC<ShipmentDetailsProps> = ({
                                                             selectedShipment,
                                                             setSelectedDewar,
                                                             sx = {},
                                                         }) => {
    const [localSelectedDewar, setLocalSelectedDewar] = React.useState<Dewar | null>(null);
    const [isAddingDewar, setIsAddingDewar] = React.useState<boolean>(false);
    const [newDewar, setNewDewar] = React.useState<Partial<Dewar>>({
        dewar_name: '',
        tracking_number: '',
    });

    const totalPucks = selectedShipment.dewars.reduce((acc, dewar) => acc + (dewar.number_of_pucks || 0), 0);
    const totalSamples = selectedShipment.dewars.reduce((acc, dewar) => acc + (dewar.number_of_samples || 0), 0);

    const handleDewarSelection = (dewar: Dewar) => {
        if (setSelectedDewar) {
            const newSelection = localSelectedDewar?.id === dewar.id ? null : dewar;
            setLocalSelectedDewar(newSelection);
            setSelectedDewar(newSelection);
        }
    };

    const handleDeleteDewar = () => {
        if (localSelectedDewar) {
            const confirmed = window.confirm('Are you sure you want to delete this dewar?');
            if (confirmed) {
                const updatedDewars = selectedShipment.dewars.filter(dewar => dewar.tracking_number !== localSelectedDewar.tracking_number);
                console.log('Updated Dewars:', updatedDewars);
                setLocalSelectedDewar(null);
            }
        }
    };

    const handleNewDewarChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        const {name, value} = e.target;
        setNewDewar((prev) => ({
            ...prev,
            [name]: value,
        }));
    };

    const handleAddDewar = async () => {
        if (selectedShipment && newDewar.dewar_name) {
            try {
                const newDewarToPost: Dewar = {
                    dewar_name: newDewar.dewar_name.trim() || 'Unnamed Dewar',
                    number_of_pucks: newDewar.number_of_pucks ?? 0,
                    number_of_samples: newDewar.number_of_samples ?? 0,
                    return_address: selectedShipment.return_address,
                    contact_person: selectedShipment.contact_person,
                    status: 'In preparation',
                    shippingStatus: 'not shipped',
                    arrivalStatus: 'not arrived',
                    qrcode: newDewar.qrcode || 'N/A',
                };

                await DefaultService.createDewarDewarsPost(newDewarToPost);
                setIsAddingDewar(false);
                setNewDewar({dewar_name: '', tracking_number: ''});
            } catch (error) {
                alert("Failed to add dewar. Please try again.");
                console.error("Error adding dewar:", error);
            }
        } else {
            alert('Please fill in the Dewar Name');
        }
    };

    return (
        <Box sx={{...sx, padding: 2, textAlign: 'left'}}>
            {!localSelectedDewar && !isAddingDewar && (
                <Button
                    variant="contained"
                    onClick={() => setIsAddingDewar(true)}
                    sx={{marginBottom: 2}}
                >
                    Add Dewar
                </Button>
            )}

            {isAddingDewar && (
                <Box sx={{marginBottom: 2, width: '20%'}}>
                    <Typography variant="h6">Add New Dewar</Typography>
                    <TextField
                        label="Dewar Name"
                        name="dewar_name"
                        value={newDewar.dewar_name}
                        onChange={handleNewDewarChange}
                        fullWidth
                        sx={{marginBottom: 2}}
                    />
                    <Button variant="contained" color="primary" onClick={handleAddDewar} sx={{marginRight: 2}}>
                        Save Dewar
                    </Button>
                    <Button variant="outlined" color="secondary" onClick={() => setIsAddingDewar(false)}>
                        Cancel
                    </Button>
                </Box>
            )}

            <Typography variant="h5">{selectedShipment.shipment_name}</Typography>
            <Typography variant="body1" color="textSecondary">Main contact person: {`${selectedShipment.contact_person[0].firstname} ${selectedShipment.contact_person[0].lastname}`}</Typography>
            <Typography variant="body1">Number of Pucks: {totalPucks}</Typography>
            <Typography variant="body1">Number of Samples: {totalSamples}</Typography>
            <Typography variant="body1">Shipment Date: {selectedShipment.shipment_date}</Typography>

            {localSelectedDewar && !isAddingDewar && (
                <DewarDetails
                    dewar={localSelectedDewar}
                    trackingNumber={localSelectedDewar.tracking_number || ''}
                    setTrackingNumber={(value) => {
                        setLocalSelectedDewar((prev) => prev ? {...prev, tracking_number: value} : prev);
                    }}
                    contactPersons={selectedShipment.contact_person}
                    returnAddresses={selectedShipment.return_address}
                />
            )}

            <Stack spacing={1}>
                {selectedShipment.dewars.map((dewar) => (
                    <Button
                        key={dewar.id}
                        onClick={() => handleDewarSelection(dewar)}
                        variant="outlined"
                        sx={{
                            display: 'flex',
                            justifyContent: 'space-between',
                            alignItems: 'center',
                            padding: 1,
                            border: '1px solid #ccc',
                            borderRadius: 1,
                            backgroundColor: localSelectedDewar?.id === dewar.id ? '#f0f0f0' : '#fff',
                        }}
                    >
                        <Box
                            sx={{
                                display: 'flex',
                                alignItems: 'center',
                                justifyContent: 'center',
                                marginRight: 2,
                            }}
                        >
                            {dewar.qrcode ? (
                                <QRCode value={dewar.qrcode} size={70}/>
                            ) : (
                                <Box
                                    sx={{
                                        width: 70,
                                        height: 70,
                                        display: 'flex',
                                        alignItems: 'center',
                                        justifyContent: 'center',
                                        border: '1px dashed #ccc',
                                        borderRadius: 1,
                                        color: 'text.secondary'
                                    }}
                                >
                                    <Typography variant="body2">No QR Code</Typography>
                                </Box>
                            )}
                        </Box>

                        <Box sx={{flexGrow: 1, marginRight: 0}}>
                            <Typography variant="body1">{dewar.dewar_name}</Typography>
                            <Typography variant="body2">Number of Pucks: {dewar.number_of_pucks || 0}</Typography>
                            <Typography variant="body2">Number of Samples: {dewar.number_of_samples || 0}</Typography>
                            <Typography variant="body2">Tracking Number: {dewar.tracking_number}</Typography>
                        </Box>

                        <Box sx={{
                            flexGrow: 1,
                            display: 'flex',
                            alignItems: 'center',
                            flexDirection: 'row',
                            justifyContent: 'space-between'
                        }}>
                            <CustomStepper dewar={dewar} />
                            {localSelectedDewar?.tracking_number === dewar.tracking_number && (
                                <Button
                                    onClick={handleDeleteDewar}
                                    color="error"
                                    sx={{
                                        minWidth: '40px',
                                        height: '40px',
                                        marginLeft: 2,
                                        padding: 0,
                                        alignSelf: 'center',
                                    }}
                                >
                                    <DeleteIcon/>
                                </Button>
                            )}
                        </Box>
                    </Button>
                ))}
            </Stack>
        </Box>
    );
};

export default ShipmentDetails;